/**
 * Copyright (C) 2010 <dueni(at)gmx.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.java.dev.faces.chain;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.faces.context.FacesContext;

import org.apache.commons.chain.impl.ContextBase;
import org.apache.commons.chain.web.faces.FacesWebContext;

import net.java.dev.faces.chain.handler.CommandHandler;

;

/**
 * Extension of {@link FacesWebContext} suitable for use in JSF applications.
 * The abstract methods are mapped to the appropriate collections of the
 * underlying <code>FacesContext</code> instance that is passed to the
 * constructor (or the initialize method).
 * 
 */
public class FacesChainContext extends ContextBase {

	private static final long serialVersionUID = 4209805904016016964L;

	private FacesContext jsf;

	/**
	 * The outcome to pass back to the JSF action e.g. for navigation.
	 */
	private Object outcome;

	/** List of executed commands maintained by the {@link FacesChainCommand} adapter. */
	private List<String> executedCommands;

	/** Key on which the current FacesChainContext may be stored on request scope. */
	public static final String FACES_CHAIN_CONTEXT_REQ_KEY = "facesChainContext";

	/**
	 * Default constructor - suitable e.g. for testing.
	 */
	public FacesChainContext() {
		this(null);
	}

	/**
	 * Construct a FacesChainContext to keep the passed FacesContext. If null is
	 * passed in, {@link FacesContext#getCurrentInstance()} will be called to
	 * aquire the current FacesContext.
	 * 
	 * @param jsf
	 *          current FacesContext or null to use
	 *          FacesContext.getCurrentInstance() instead.
	 */
	public FacesChainContext(FacesContext jsf) {
		super();
		if (jsf == null) {
			this.jsf = FacesContext.getCurrentInstance();
		} else {
			this.jsf = jsf;
		}
		executedCommands = new ArrayList<String>();
	}

	/**
	 * Return a mutable <code>Map</code> that maps application scope attribute
	 * names to their values.
	 * 
	 * @return Application scope Map.
	 */
	public final Map<String, Object> getApplicationScope() {
		return jsf.getExternalContext().getApplicationMap();
	}

	/**
	 * Return an immutable <code>Map</code> that maps header names to the first
	 * (or only) header value (as a String). Header names must be matched in a
	 * case-insensitive manner.
	 * 
	 * @return Header values Map.
	 */
	public final Map<String, String> getHeader() {
		return jsf.getExternalContext().getRequestHeaderMap();
	}

	/**
	 * Return an immutable <code>Map</code> that maps header names to the set of
	 * all values specified in the request (as a String array). Header names must
	 * be matched in a case-insensitive manner.
	 * 
	 * @return Header values Map.
	 */
	public final Map<String, String[]> getHeaderValues() {
		return jsf.getExternalContext().getRequestHeaderValuesMap();
	}

	/**
	 * Return an immutable <code>Map</code> that maps context application
	 * initialization parameters to their values.
	 * 
	 * @return Initialization parameter Map.
	 */
	public final Map<?, ?> getInitParam() {
		return jsf.getExternalContext().getInitParameterMap();
	}

	/**
	 * Return an immutable <code>Map</code> that maps request parameter names to
	 * the first (or only) value (as a String).
	 * 
	 * @return Request parameter Map.
	 */
	public final Map<String, String> getParam() {
		return jsf.getExternalContext().getRequestParameterMap();
	}

	/**
	 * Return an immutable <code>Map</code> that maps request parameter names to
	 * the set of all values (as a String array).
	 * 
	 * @return Request parameter Map.
	 */
	public final Map<String, String[]> getParamValues() {
		return jsf.getExternalContext().getRequestParameterValuesMap();
	}

	/**
	 * Return an immutable <code>Map</code> that maps cookie names to the set of
	 * cookies specified in the request.
	 * 
	 * @return Map of Cookies.
	 * @since Chain 1.1
	 */
	public final Map<String, Object> getCookies() {
		return jsf.getExternalContext().getRequestCookieMap();
	}

	/**
	 * Return a mutable <code>Map</code> that maps request scope attribute names
	 * to their values.
	 * 
	 * @return Request scope Map.
	 */
	public final Map<String, Object> getRequestScope() {
		return jsf.getExternalContext().getRequestMap();
	}

	/**
	 * Return a mutable <code>Map</code> that maps session scope attribute names
	 * to their values.
	 * 
	 * @return Session scope Map.
	 */
	public final Map<String, Object> getSessionScope() {
		return jsf.getExternalContext().getSessionMap();
	}

	/**
	 * Return the outcome to be passed to the calling action handler.&nbsp;For
	 * normal <code>action</code> mapping only String is supported as action
	 * outcome. In some cases it might be useful to allow any Object as outcome -
	 * therefore the return type is Object.
	 * 
	 * @return outcome String.
	 */
	public Object getOutcome() {
		return outcome;
	}

	/**
	 * Set the outcome that {@link CommandHandler#execute()} will pass back to the
	 * JSF action.
	 * 
	 * @param outcome
	 *          the outcome to pass back to the action.
	 */
	public void setOutcome(Object outcome) {
		this.outcome = outcome;
	}

	/**
	 * Return the list to track executed commands.
	 * 
	 * @return the list to track executed commands.
	 */
	public List<String> getExecutedCommands() {
		return executedCommands;
	}

}
