/**
 * Copyright (C) 2010 <dueni(at)gmx.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.java.dev.faces.chain;

import java.io.IOException;
import java.lang.reflect.Constructor;
import java.net.URL;
import java.util.Arrays;
import java.util.Properties;

import javax.el.ELContext;
import javax.faces.context.FacesContext;

import org.apache.commons.chain.Command;

import net.java.dev.faces.chain.handler.CommandHandler;

/**
 * <code>FacesChainFactory</code> allows to use custom {@link FacesChainFactory}
 * , {@link FacesChainContext} and/or {@link CommandHandler} by providing the
 * customized implementations and a factory configuration file named
 * {@value #CONFIG_FILE_NAME} on the class-path.
 * <p>
 * Valid configuration options in {@value #CONFIG_FILE_NAME} are:
 * </p>
 * <dl>
 * <dt>net.java.dev.faces.chain.FacesChainFactory</dt>
 * <dd>The class name of a custom FacesChainFactory - must extend
 * {@link FacesChainFactory}.</dd>
 * <dt>net.java.dev.faces.chain.FacesChainContext</dt>
 * <dd>The class name of a custom FacesChainContext - the custom
 * FacesChainContext implementation must extend {@link FacesChainContext} and
 * provide a Constructor accepting one parameter of type {@link FacesContext}.</dd>
 * <dt>net.java.dev.faces.chain.handler.CommandHandler</dt>
 * <dd>The class name of a custom CommandHandler - the custom CommandHandler
 * implementation must extend {@link CommandHandler} and provide a Constructor
 * accepting three parameters of type {@link ELContext}, {@link String} and
 * {@link Command} in this order.</dd>
 * </dl>
 * 
 * @author dueni
 * 
 */
public class FacesChainFactory {

	/** The properties file name to configure this factory. */
	private static final String CONFIG_FILE_NAME = "faces-chain.properties";

	/** The factory singleton instance. */
	private static FacesChainFactory factory;

	/** The constructor to create new instances of FacesChainContext. */
	private Constructor<?> facesChainContextCtr;

	/** The constructor to create new instances of CommandHandler. */
	private Constructor<?> commandHandlerCtr;

	/**
	 * Return the singleton FacesChainFactory instance.
	 * 
	 * @return the singleton FacesChainFactory instance.
	 */
	public final static FacesChainFactory getInstance() {
		if (factory == null) {
			ClassLoader cl = Thread.currentThread().getContextClassLoader();
			Properties config = loadFactoryConfig(cl);

			String factoryClass = config.getProperty(FacesChainFactory.class.getName(),
					FacesChainFactory.class.getName());
			factory = (FacesChainFactory) newInstance(cl, factoryClass);
			factory.initialize(cl, config);
		}
		return factory;
	}

	/**
	 * Return a new instance of FacesChainContext (or a custom implementation of
	 * it) initialized with the given FacesContext.
	 * 
	 * @param jsf
	 *          the FacesContext to pass to the FacesChainContext constructor.
	 * @return a new instance of FacesChainContext,
	 */
	public FacesChainContext createFacesChainContext(FacesContext jsf) {
		try {
			return (FacesChainContext) facesChainContextCtr.newInstance(jsf);
		} catch (Exception e) {
			throw new IllegalStateException("This should never happpen!", e);
		}
	}

	/**
	 * Return a new instance of CommandHandler (or a custom implementation of it)
	 * initialized with the given ELContext, String and Command.
	 * 
	 * @param elContext
	 *          elContext is the first parameter on the CommandHandler
	 *          constructor.
	 * @param name
	 *          name is the second parameter on the CommandHandler constructor.
	 * @param command
	 *          command is the third parameter on the CommandHandler constructor.
	 * @return a new instance of CommandHandler.
	 */
	public CommandHandler createCommandHandler(ELContext elContext, String name, Command command) {
		try {
			return (CommandHandler) commandHandlerCtr.newInstance(elContext, name, command);
		} catch (Exception e) {
			throw new IllegalStateException("This should never happpen!", e);
		}
	}

	/* -------------------------------------------------------------------- */

	/**
	 * Initialize FacesContextFactory according to the {@value #CONFIG_FILE_NAME}
	 * settings.
	 * 
	 */
	protected void initialize(ClassLoader cl, Properties config) {
		String className = config.getProperty(FacesChainContext.class.getName(),
				FacesChainContext.class.getName());
		facesChainContextCtr = getConstructor(cl, className, FacesContext.class);

		className = config.getProperty(CommandHandler.class.getName(), CommandHandler.class.getName());
		commandHandlerCtr = getConstructor(cl, className, ELContext.class, String.class, Command.class);
	}

	/**
	 * Return the Constructor for the named class accepting the given
	 * parameterTypes.
	 * 
	 * @param cl
	 *          ClassLoader to load Class.
	 * @param className
	 *          name of the class to load.
	 * @param parameterTypes
	 *          parameter types to lookup the wanted constructor.
	 * @return constructor for parameter types.
	 * @throws IllegalArgumentException
	 *           if class or wanted constructor cannot be found.
	 */
	protected static Constructor<?> getConstructor(ClassLoader cl, String className,
			Class<?>... parameterTypes) {
		try {
			Class<?> clazz = cl.loadClass(className);
			return clazz.getConstructor(parameterTypes);
		} catch (Exception e) {
			throw new IllegalArgumentException("Could not find constructor supporting arguments "
					+ Arrays.asList(parameterTypes) + " on class '" + className + "'!", e);
		}
	}

	/**
	 * Return new instance of the named class.
	 * 
	 * @param cl
	 *          the ClassLoader to load the class.
	 * @param className
	 *          the name of the class to return an instance of.
	 * @return new instance of the named class.
	 * @throws IllegalArgumentException
	 *           if the given class could not be loaded or no instance could be
	 *           created.
	 */
	protected static Object newInstance(ClassLoader cl, String className) {
		Object result = null;
		try {
			Class<?> clazz = cl.loadClass(className);
			result = clazz.newInstance();
		} catch (Exception e) {
			throw new IllegalArgumentException("Could not instanciate '" + className + "'!", e);
		}
		return result;
	}

	/**
	 * Return a Properties object containing the optional configuration settings
	 * for FacesChainFactory - the Properties object is empty if no configuration
	 * was found.
	 * 
	 * @param cl
	 *          the class loader used to lookup for the {@value #CONFIG_FILE_NAME}
	 *          .�
	 * @return a Properties object containing the actual configuration - might be
	 *         empty if no configuration file is present.
	 */
	protected static Properties loadFactoryConfig(ClassLoader cl) {
		URL propFile = cl.getResource(CONFIG_FILE_NAME);
		Properties config = new Properties();
		if (propFile != null) {
			try {
				config.load(propFile.openStream());
			} catch (IOException e) {
				throw new IllegalArgumentException("Invalid content (not valid properties file?) in: "
						+ propFile, e);
			}
		}
		return config;
	}

}
